<?php
//  Database access class
//
abstract class AdministrateDB {
	
	public function __construct() {
		$this->last_query	= "";
	}
	
	//  Executes a query
	abstract public function query($qry);
	
	//  Executes a query and get result set
	abstract public function query_fetch($qry);
	
	//  Prepare statement
	abstract public function prepare($qry, $param_types = "");
	
	//  Execute prepared
	abstract public function prepared_exec($prep_statement, $params = array());
	
	//  Execute prepared and get result set
	abstract public function prepared_fetch($prep_statement, $params = array());
	
	//  Gets the number of records returned by a query
	abstract public function get_last_num_records();
	
	//  Get last auto-generated insert id
	abstract public function get_last_insert_id();
	
	//  Get last error
	abstract public function get_last_error();
	
	/*
	
	Value Formatters
	
	*/	
	abstract public function format_string($val);
	abstract public function format_date($val);
	
	/* 
	
	Query Helpers
	
	NOTE:
	This is not an "abstraction" of SQL,
	it's just a way to simplify the most common queries.
	
	*/
	
	//  Formats a select query
	public function select($table, $fields = "*", $criteria = false, $order = false, $limit = false) {
		
		$f_cols = $this->format_val_list($cols);
		$f_where  = $criteria ? "WHERE ".$this->format_expression_list($criteria, "=",'AND') : "";
		$f_order  = $order ? "ORDR BY {$order}" : "";
		$f_limit  = $limit ? "LIMIT {$limit}" : "";
		
		$q = "SELECT {$f_cols} FROM {$table} {$f_where} {$f_order} {$f_limit}";
		
		// Execute query
		return $this->query_fetch($q);
	}
	
	//  Formats an insert
	public function insert($table, $field_list) {
		$f_cols = $this->format_val_list( array_keys($field_list) );
		$f_vals = $this->format_val_list( array_values($field_list), true );
		
		$q = "INSERT INTO {$table} ";
		$q .= " ( {$f_cols} ) VALUES ( {$f_vals} )";
		
		// Execute query
		return $this->query($q);
	}
	
	//  Formats an update
	public function update($table, $data_fields, $criteria = "") {
		
		$f_eq_clause = $this->format_expression_list($data_fields, "=",',');
		$f_where  = $criteria ? "WHERE ".$this->format_expression_list($criteria, "=",'AND') : "";
		
		$q = "UPDATE {$table} SET {$f_eq_clause} {$f_where}";
		
		// Execute query
		return $this->query($q);
	}
	
	// Format value
	public function format_val($val) {
		if (is_string($val)) {
			$f_str = $this->format_string($val);
			return "'{$f_str}'";
		}
		return $val;
	}
	
	//  Formats a column list
	public function format_val_list($val_list, $format_vals = false) {
		if(!is_array($val_list)) {
			return $val_list;
		}
		
		$f_val_list = "";
		
		foreach ($val_list as $val) {
			$f_val = $format_vals ? $this->format_val($val) : $val;
			$f_val_list .= ($f_val_list=="") ? $f_val :",".$f_val;
		}
		
		return $f_val_list;
	}
	
	//  Formats an expression list
	public function format_expression_list($val_list, $operator="=", $separator=",") {
		if(!is_array($val_list)) {
			return $val_list;
		}
		
		$f_clause = "";
		
		foreach ($val_list as $col => $val) {
			$f_exp = $col." ".$operator." ".$this->format_val($val);
			$f_clause .= ($f_clause=="") ? $f_exp : $separator.' '.$f_exp;
		}
		
		return $f_clause;
	}
	
	/*
	
	Internal event logging
	
	*/
	
	//  Save last query
	protected function _set_last_query($str) {
		$this->last_query = $str;
	}
	
	//  Get last query
	protected function _get_last_query() {
		return $this->last_query;
	}
	
	//  Log an event
	protected function _log($user_message = "") {
		$message = "DB: ";
		
		if($user_message != "")  {
			$message .= $user_message . " \n";
		}
		/*if($this->_get_last_query() != "")  {
			$message .= "Last Query: " . $this->_get_last_query();
		}*/
		
		Administrate::log()->notice($message);
	}
	
	//  Log an error
	protected function _warn($user_message = "") {
		$message = "DB: ";
		
		if($user_message != "")  {
			$message .= $user_message . " \n";
		}		
		if($this->get_last_error() != "")  {
			$message .= "Last Error: " . $this->get_last_error() . " \n";
		}
		if($this->_get_last_query() != "")  {
			$message .= "Last Query: " . $this->_get_last_query();
		}
			
		Administrate::log()->warning($message);
	}
	
}
?>
