<?php
//  Require API objects
//
require_once('AdministrateBaseAPI.php');

require_once('AdministrateCache.php');
require_once('AdministrateCacheAlgorithm.php');

require_once('objects/AdministrateAPIObject.php');
require_once('objects/AdministrateAPIEvent.php');
require_once('objects/AdministrateAPIEventSession.php');
require_once('objects/AdministrateAPIEventPrice.php');
require_once('objects/AdministrateAPICourse.php');
require_once('objects/AdministrateAPICourseSubcategory.php');
require_once('objects/AdministrateAPICourseCategory.php');
require_once('objects/AdministrateAPILocation.php');
require_once('objects/AdministrateAPIOrder.php');
require_once('objects/AdministrateAPIUser.php');
require_once('objects/AdministrateAPIDelegate.php');
require_once('objects/AdministrateAPICartEntry.php');
require_once('objects/AdministrateAPIOrderDetailsAltAddress.php');
require_once('objects/AdministrateAPIDelegateCartEntry.php');
require_once('objects/AdministrateAPIOrderDetails.php');
require_once('objects/AdministrateAPIOrderPayment.php');

// Administrate API
//
class AdministrateAPI extends AdministrateBaseAPI {
	private $_cache_algorithm = null;
	private $_internal_course_cache = array();
	private $_internal_event_cache = array();

	//  Get singleton
	public static function singleton() {
		static $api = null;

		if( is_null($api) ) {
			$api = new AdministrateAPI();
		}
		return $api;
	}

	// Set caching
	public function set_cache($cache=null) {
		if(!$cache){
			$this->_cache_algorithm = null;
			$this->_log("Caching is disabled");
			return;
		}
		$this->_cache_algorithm = new AdministrateCacheAlgorithm($cache);
		$this->_log("Caching is enabled");
	}

	// Make API call or retreive objects from cache
	public function raw_call($method, $args=array(), $cache_call=true) {
		if( is_null($this->_cache_algorithm) || !$cache_call ) {
			$this->_log("No cache for this call");
			return parent::raw_call($method, $args);
		}

		if( ($method == "getEventByID") && is_array($args) && array_key_exists("InvoiceLocation", $args) ) {
			// The argument 'InvoiceLocation' can return different data for each visitor,
			// it should not be cached.
			return parent::raw_call($method, $args);
		}

		if($method == 'getEvents') {
			if(array_key_exists('Filter', $args)) {
				if(array_key_exists('IncludeCurrentOnline', $args['Filter']) && $args['Filter']['IncludeCurrentOnline'] == '') {
					$args['Filter']['IncludeCurrentOnline'] = false;
				}
				if(array_key_exists('IncludeSessions', $args['Filter']) && $args['Filter']['IncludeSessions'] == 1) {
					$args['Filter']['IncludeSessions'] = true;
				}
			}
		}

		// Find cached objects
		$cache_hit = $this->_cache_algorithm->find_objects($method, $args);

		// Make API call if no objects found
		if( is_null($cache_hit) ) {
			$this->_log("Cache miss");
			$api_result = parent::raw_call($method, $args);
			$this->_cache_algorithm->record_objects($method, $args, $api_result);
			return $api_result;
		}

		// Cache hit
		$this->_log("Cache hit");
		return $cache_hit;
	}

	/*
	* Event API calls
	*/

	// Get a specific event
	public function get_event($eid, $cache_call=true) {
		$this->_log('get_event ID ' . var_export($eid, true));

		if( !is_array($eid) ) {
			$eid = array( $eid );
		}

		$call_result = $this->raw_call('getEventByID', $eid, $cache_call);

		$event = null;
		if( $call_result ) {
			$event = new AdministrateAPIEvent($call_result);
		}
		else {
			$this->_warn('get_event error');
			return null;
		}

		return $event;
	}

	public function get_learning_track_list() {
		return $this->raw_call('get_learning_tracks');
	}

	public function get_learning_track($track_id) {
		return $this->raw_call('get_learning_track', $track_id, false);
	}

	public function get_mapped_items($course_id, $location_id) {
		return $this->raw_call(
			'get_mapped_items',
			array(
				'course_id' => $course_id,
				'location_id' => $location_id
			)
		);
	}

	public function get_mapped_item($item_id) {
		return $this->raw_call(
			'get_mapped_item',
			$item_id
		);
	}

	// Get event list
	public function get_event_list($params = array()) {

		$this->_log('get_event_list');

		// Set default price option
		if( !array_key_exists('IncludePrices', $params) ) {
			$params = array_merge( array( 'IncludePrices' => true ), $params );
		}

		$raw_events = array();
		if(array_key_exists('CourseCode', $params)
			&& is_array($params['CourseCode'])
			&& count($params['CourseCode'])
		){
			foreach($params['CourseCode'] as $course_code) {

				$course_params = array_merge(
					$params,
					array('CourseCode' => $course_code)
				);

				$course_call = $this->raw_call('getEvents', array('Filter' => $course_params));
				if(!isset($course_call->EventList)) {
					continue;
				}
				foreach($course_call->EventList as $event) {
					array_push($raw_events, $event);
				}
			}
		} else {

			// Make API call
			$call_result = $this->raw_call('getEvents', array('Filter' => $params));
			if(property_exists($call_result, 'EventList')) {
				$raw_events = $call_result->EventList;
			} else {
				return array();
			}
		}

		if( !$raw_events ) {
			// I'm leaving this commented out. I don't think having no events should be
			// a warning, since it's perfectly valid under normal operation.
			// $this->_warn('get_event_list error');
			return array();
		}

		// Loop through events
		$events = array();
		$i = 0;

		foreach ($raw_events as $result_event) {
			if($result_event->IsProvisional) {
				continue;
			}

			// Initialize the event object
			$event = new AdministrateAPIEvent($result_event);
			$this->_internal_event_cache[$event->get_id()] = $event;

			//  Check subcategory ID
			if (array_key_exists('CourseSubCategoryID', $params)) {
				$has_subcategory = false;

				if($event->is_in_subcategory($params['CourseSubCategoryID'])) {
					$has_subcategory = true;
				}

				if(!$has_subcategory) {
					continue;
				}
			}

			// Add this event to the array
			$events[$event->get_start_date() . '-' . $i] = $event;
			$i++;
		}

		// Sort and return events
		ksort($events);
		return $events;
	}

	//  Get number of event delegates
	public function get_num_delegates($event_id) {
		$this->_log('get_num_delegates');
		return $this->raw_call('getNumDelegates', array($event_id));
	}

	//  Get unique locations
	public function get_events_locations($events) {
		$locations = array();
		foreach ($events as $event) {

			$location = $event->get_location();

			// Strip non-alphabet characters from array key
			// NOTE: It's not clear why we need this ( using the key in GET requests? )
			$key = $this->_strip_non_alphabet($location);

			if (!isset($locations[$key])) {
				$locations[$key] = $event->get_location();
			}
		}
		ksort($locations);
		return $locations;
	}

	//  Get courses from events
	public function get_events_courses($events) {

		//  Loop through events and add unique courses
		$courses = array();
		foreach ($events as $event) {
			$course_code = $event->get_course_code();
			if (!isset($courses[$course_code])) {
				$courses[$course_code] = $this->get_course($course_code);
			}
		}

		// Sort courses by code
		ksort($courses);

		//  Return the courses sorted by code
		return $courses;

	}

	// Strip non-alphabet characters
	private function _strip_non_alphabet($str) {
		return preg_replace('/[^a-z]/i', '', strtolower($str));
	}

	/*
	* Course API calls
	*/
	//  Get a course by code
	public function get_course($code) {
		if(array_key_exists($code, $this->_internal_course_cache)) {
			return $this->_internal_course_cache[$code];
		}
		$this->_log('get_course, ' . $code);
		$call_result = $this->raw_call('getCourseByCode', array($code));

		$course = null;
		if(!$call_result) {
			$this->_warn('get_course error');
			$course = new AdministrateAPICourse();
		}

		$course = new AdministrateAPICourse($call_result);

		$this->_internal_course_cache[$code] = $course;
		return $course;
	}

	//  Get courses
	public function get_course_list($params = array()) {
		$this->_log('get_course_list');

		//  Make API call
		$call_result = $this->raw_call('getCourses', $params);

		if( !$call_result ) {
			$this->_warn('get_course_list error');
			return array();
		}

		if( !isset($call_result->CourseList) ) {
			// Empty course list but not an error
			return array();
		}

		//  Loop through events
		$courses = array();

		foreach ($call_result->CourseList as $course) {

			//  Initialize the event object
			$course = new AdministrateAPICourse($course);
			$this->_internal_course_cache[$course->get_code()] = $course;

			//  Check category ID
			if (array_key_exists('CourseCategoryID', $params)) {
				if( !is_array($params['CourseCategoryID']) ) {
					$params['CourseCategoryID'] = array( $params['CourseCategoryID'] );
				}
				$has_category = false;

				foreach($params['CourseCategoryID'] as $id) {
					if($this->_categories_contain_id(false, $id, $course->get_categories())) {
						$has_category = true;
					}
				}
				if(!$has_category) {
					continue;
				}
			}

			//  Check subcategory ID
			if (array_key_exists('CourseSubCategoryID', $params)) {
				if( !is_array($params['CourseSubCategoryID']) ) {
					$params['CourseSubCategoryID'] = array( $params['CourseSubCategoryID'] );
				}
				$has_subcategory = false;

				foreach($params['CourseSubCategoryID'] as $id) {
					if($this->_categories_contain_id(true, $id, $course->get_categories())) {
						$has_subcategory = true;
					}
				}
				if(!$has_subcategory) {
					continue;
				}
			}

			if (array_key_exists('CourseCode', $params) && is_array($params['CourseCode'])) {
				$has_code = false;

				foreach($params['CourseCode'] as $course_code) {
					if($course->get_code() == $course_code) {
						$has_code = true;
					}
				}
				if(!$has_code) {
					continue;
				}
			}

			// Add the course to the array
			$courses[$course->get_code()] = $course;
		}

		// Sort and return courses by code
		ksort($courses);
		return $courses;
	}

	//  Whether or not a set of categories include the desired ID
	private function _categories_contain_id($subcat_search, $category_id, $categories_list) {

		if (!$categories_list) {
			//  If no set of categories was passed, get them all
			//$categories_list = $this->get_category_list();
			return false;
		}

		//  Loop through categories
		foreach ($categories_list as $category) {
			// Loop through subcategories
			if($subcat_search) {
				foreach ($category->get_subcategories() as $subcategory) {
					//  If the subcategory matches the ID, result is positive and break loop x2
					if ($subcategory->get_id() == $category_id) {
						return true;
					}
				}
			}
			// Category search
			else if ( $category->get_id() == $category_id ) {
				return true;
			}

		}
		// No matching categorty
		return false;

	}

	//  Get categories
	public function get_category_list( $event_filter = array() ) {

		if(is_array($event_filter)) {
			$event_filter = array_merge(
				$event_filter,
				array(
					'IncludeEmpty' => true
				)
			);
		} else {
			$event_filter = array('IncludeEmpty' => true);
		}

		$this->_log('get_category_list, event_filter = '.var_export($event_filter,true) );

		//  Get API objects
		$call_result = $this->raw_call('getCategories', array());

		if(!$call_result) {
			$this->_warn('get_categories error');
			return array();
		}


		$categories = array();

		foreach ($call_result->CategoryList as $category) {
			array_push($categories, new AdministrateAPICourseCategory($category));
		}

		if( array_key_exists("IncludeEmpty", $event_filter ) && $event_filter["IncludeEmpty"] ) {
			// Return all categories
			usort($categories, array("AdministrateAPI","category_compare") );
			return $categories;
		}

		if(array_key_exists('CourseCode', $event_filter) && !is_array($event_filter['CourseCode'])) {
			$event_filter['CourseCode'] = array_map('trim', explode(',', $event_filter['CourseCode']));
		}

		$event_list = $this->get_event_list( $event_filter );
		$categories_filtered = array();

		if(!$event_list) {
			return array();
		}

		// Return non-empty categories
		foreach( $categories as $cat ) {
			$category_empty = true;

			foreach( $event_list as $event ) {
				$event_categories = $event->get_categories();

				if( !$event_categories ) {
					continue;
				}

				foreach( $event_categories as $event_cat ) {
					if( $cat->get_id() == $event_cat->get_id() ) {
						array_push($categories_filtered, $cat);
						$category_empty = false;
						break;
					}
				}

				if( !$category_empty ) {
					break;
				}
			}
		}
		usort($categories_filtered, array("AdministrateAPI","category_compare") );
		return $categories_filtered;
	}

	public static function category_compare($cat1, $cat2)	{
		return $cat1->get_index() - $cat2->get_index();
	}

	//  Get a single category
	public function get_category( $category_id, $event_filter = array() ) {

		//  Get all categories
		$categories = $this->get_category_list( $event_filter );

		//  Loop through categories until we find the ID
		foreach ($categories as $cat) {
			if ($cat->get_id() == $category_id) {
				return $cat;
			}
		}

		// No category with given ID
		return new AdministrateAPICourseCategory();

	}

	//  Get a single subcategory
	public function get_subcategory($subcategory_id, $category_id = false, $event_filter = array()) {

		//  If no category ID is defined, fetch all categories
		if ( $category_id === false ) {
			$categories = $this->get_category_list( $event_filter );
		}
		// Fetch single category with for the given ID
		else {
			$categories = array($this->get_category($category_id));
		}

		//  Loop through categories until we find the right subcategory
		foreach ($categories as $cat)
		{
			$subcategories = $cat->get_subcategories();

			if (count($subcategories) == 0) {
				continue;
			}

			foreach ($subcategories as $subcat) {
				//  Return matching subcategory
				if ($subcat->get_id() == $subcategory_id) {
						return $subcat;
					}
				}

		}
		// No matching subcategory
		return new AdministrateAPICourseSubcategory();

	}

	/*
	* Location API calls
	*/

	//  Get locations
	public function get_location_list() {
		$this->_log("get_location_list");

		//  Make the SOAP call
		$call_result = $this->raw_call('getLocations', array());

		if(!$call_result) {
			$this->_warn("get_location_list error");
			return array();
		}

		//  Loop through locations
		$locations = array();

		foreach ($call_result->LocationList as $location) {
			array_push($locations, new AdministrateAPILocation($location));
		}
		//  Return the vetted locations
		return $locations;
	}

	/*
	* Order API calls
	*/

	//  Place an order
	public function submit_order($fields = array()) {
		$this->_log("submit_order ".var_export($fields,true));

		$prepared = AdministrateAPIOrder::prepare_api_object($fields);
		$orderNum = $this->raw_call('placeOrder', $prepared);

		return $orderNum;
	}

	//  Send an invoice
	public function send_invoice($order_num) {
		$this->_log('send_invoice for order #' . $order_num);

		// All API calls accept single arguments in the form array(argument_value)
		// The call sendInvoice is the only exception - it accepts a plain value
		// Passing an array causes a PHP error in the API.
		$call_result = $this->raw_call('sendInvoice', array($order_num));

		if( $call_result ) {
			Administrate::log()->notice( var_export($call_result,true) );
			return new AdministrateAPIEvent($call_result);
		}
		else {
			$this->_warn('send_invoice error');
			return new AdministrateAPIEvent();
		}
	}

	public function place_opps_order($order_data) {
		$order = $this->raw_call('place_opps_order', $order_data, false);
		return $order;
	}

	/*
	* User API calls
	*/

	//  Add a user
	public function add_user($fields = array()) {
		Administrate::log()->notice('add_user ' . var_export($fields,true));
		$p = AdministrateAPIUser::prepare_api_object($fields);

		// For our purposes we don't want to create a WebsiteDelegate against our user immediately,
		// as we'll populate the delegates separately with further calls.
		$p->DontCreateDelegate = true;
		$user_data = $this->raw_call('addUser', $p);

		if(!$user_data) {
			$this->_warn('add_user error');
			return new AdministrateAPIUser();
		}
		return new AdministrateAPIUser($user_data);
	}

	// Get user by email
	public function get_user_by_email($email) {
		Administrate::log()->notice('get_user_by_email, email=' . $email);

		$user = Administrate::api()->raw_call('getUserByEmail', array($email));

		if(!$user) {
			$this->_warn('get_user_by_email error');
			return new AdministrateAPIUser();
		}
		return new AdministrateAPIUser($user);
	}

	/*
	* Delegate API calls
	*/

	//  Add a delegate
	public function add_delegate($fields = array()) {
		Administrate::log()->notice('add_delegate ' . var_export($fields,true));

		$prepared_fields = AdministrateAPIDelegate::prepare_api_object($fields);
		$delegate_data = $this->raw_call('addDelegate', $prepared_fields);

		if(!$delegate_data) {
			$this->_warn('add_delegate error');
			return false;
		}

		$user = new AdministrateAPIUser($delegate_data);
		$user_delegates = $user->get_delegates();

		if(count($user_delegates) == 1) {
			return $user_delegates[0];
		} else {
			foreach ($user_delegates as $delegate) {
				// We've potentially added multiple delegates, and need to match the one we added
				// with one of the delegates that the APIUser now knows about to grab it's ID. Now
				// that emails aren't required, we need a more rigorous check to make sure we get
				// the correct $delegate.

				if (
					$delegate->get_first_name() == $fields['first_name']
					&& $delegate->get_last_name() == $fields['last_name']
					&& ($delegate->get_email() == $fields['email'] || empty($fields['email']))
				) {
					return $delegate;
				}
			}
		}
		return false;
	}

	/*
	* Payment API calls
	*/

	public function get_payment_config($provider) {
		$this->_log("get_payment_config, provider=".$provider);
		return $this->raw_call('getPaymentIntegrationConfig', array('provider'=>$provider));
	}

}
