<?php
// Cache database abstraction
//
class AdministrateCache{
	private $_db = null;
	private $_cache_key = null;

	// Cacheable methods
	private $_cached_methods = array(
		'getEventByID' => array("table" => 'cached_events', 'multiple' => false),
		'getEvents' => array("table" => 'cached_events', 'multiple' => true),
		'getCourseByCode' => array("table" => 'cached_courses', 'multiple' => false),
		'getCourses' => array("table" => 'cached_courses', 'multiple' => true),
		'getCategories' => array("table" => 'cached_categories', 'multiple' => true),
		'get_learning_tracks' => array('table' => 'cached_tracks', 'multiple' => true),
		'get_learning_track' => array('table' => 'cached_tracks', 'multiple' => false),
	);

	// Parameters of cached objects
	private $_cached_parameters = array(
		'cached_events' => array(
			'object_lifetime' => 0,
			'id_column' => 'EventID',
			'list_name' => 'EventList',
			'prep_select' => null,
			'prep_insert' => null,
			'prep_multi_select' => null,
		),
		'cached_courses' => array(
			'object_lifetime' => 0,
			'id_column' => 'Code',
			'list_name' => 'CourseList',
			'prep_select' => null,
			'prep_insert' => null,
			'prep_multi_select' => null,
		),
		'cached_categories' => array(
			'object_lifetime' => 0,
			'id_column' => 'CategoryID',
			'list_name' => 'CategoryList',
			'prep_select' => null,
			'prep_insert' => null,
			'prep_multi_select' => null,
		),
		'cached_tracks' => array(
			'object_lifetime' => 0,
			'id_column' => 'id',
			'list_name' => null,
			'prep_select' => null,
			'prep_insert' => null,
			'prep_multi_select' => null,
		)
	);
	private $_cache_lifetime = 0;
	private $_prep_call_select = null;
	private $_prep_call_insert = null;

	// Construct cache
	public function __construct($database, $table_prefix, $cache_key, $t_expire = 3600) {
		$this->_db = $database;
		$this->_table_prefix = $table_prefix;
		$this->_cache_key = $cache_key;
		$this->_set_expire_time($t_expire);

		// Prepared queries
		//
		foreach($this->_cached_parameters as $entity => $params) {
			$object_lifetime = $params["object_lifetime"];
			$t = $this->_table_prefix.$entity;

			$query = "
				SELECT value
				FROM {$t}
				WHERE
					api_key = '{$this->_cache_key}'
					AND id = (?)
					AND cached_at >= DATE_SUB(NOW(), INTERVAL {$object_lifetime} SECOND)
					ORDER BY
					cached_at DESC
					LIMIT 1
			";
			$this->_cached_parameters[$entity]['prep_select'] = $this->_db->prepare($query, "s");

			$query = "
				SELECT id, value
				FROM {$t}
				WHERE
					api_key = '{$this->_cache_key}'
					AND FIND_IN_SET(id, (?))
					AND cached_at >= DATE_SUB(NOW(), INTERVAL {$object_lifetime} SECOND)
				GROUP BY id
			";
			$this->_cached_parameters[$entity]['prep_multi_select'] = $this->_db->prepare($query, "s");

			$query = "
				INSERT INTO {$t}
				SET
					api_key = '{$this->_cache_key}',
					id = (?),
					value = (?)
			";
			$this->_cached_parameters[$entity]['prep_insert'] = $this->_db->prepare($query, "ss");
		}

		$query = "
			SELECT result_ids
			FROM {$this->_table_prefix}cached_calls
			WHERE
				api_key = '{$this->_cache_key}'
				AND method = (?)
				AND args = (?)
			ORDER BY
				cached_at DESC
			LIMIT 1;
		";
		$this->_prep_call_select = $this->_db->prepare($query, "ss");

		$query = "
			INSERT INTO
				{$this->_table_prefix}cached_calls
			SET
				api_key = '{$this->_cache_key}',
				method = (?),
				args = (?),
				result_ids = (?)
		";
		$this->_prep_call_insert = $this->_db->prepare($query, "sss");

	}

	// Set cache expire time
	private function _set_expire_time($t, $t_object = array( "cached_events" => 300 )) {

		// Set object expire times
		foreach( array_keys($this->_cached_parameters) as $name ) {
			if( array_key_exists($name, $t_object) ) {
				$this->_cached_parameters[$name]['object_lifetime'] = $t_object[$name];
			}
			else {
				$this->_cached_parameters[$name]['object_lifetime'] = $t;
			}
		}
		$this->_cache_lifetime = $t;

		// Pick the longest object expire time as cache lifetime
		foreach( $this->_cached_parameters as $name => $params ) {
			if( $params['object_lifetime'] > $this->_cache_lifetime ) {
				$this->_cache_lifetime = $params['object_lifetime'] + 1;
			}
		}
	}

	// Check if call is chaceable
	public function is_call_cacheable($method) {
		return array_key_exists($method, $this->_cached_methods);
	}

	// Check if call results in multiple objects
	public function is_result_multiple($method) {
		return $this->_cached_methods[$method]['multiple'];
	}

	// Get call return type
	public function get_call_type($method)  {
		return $this->_cached_methods[$method]["table"];
	}

	// Get call caching parameters
	public function get_call_parameters($call_type)  {
		return $this->_cached_parameters[$call_type];
	}

	// Fetch single object form database
	public function fetch_single_object($method, $id) {
		$entity = $this->_cached_methods[$method]["table"];

		$data = $this->_db->prepared_fetch(
			$this->_cached_parameters[$entity]["prep_select"],
			array( $this->_db->format_string($id) )
		);

		// JSON decode and return
		if (is_array($data) && array_key_exists('value', $data)) {
			return json_decode($data['value']);
		}
		// Cache miss
		return null;
	}

	// Fetch multiple objects form database
	public function fetch_multiple_objects($method, $id_array, $entity_params) {
		if($id_array=="") {
			// Bail out early if no objects.
			return null;
		}

		$entity = $this->_cached_methods[$method]["table"];

		$rows = $this->_db->prepared_fetch(
			$this->_cached_parameters[$entity]["prep_multi_select"],
			array( $id_array )
		);

		$object_count = $this->_db->get_last_num_records();

		if ($object_count > 0) {
			if($object_count == 1) {
				$rows = array($rows);
			}

			// JSON decode items
			$items = array();
			foreach($rows as $r) {
				$items[] = json_decode($r['value']);
			}

			if(!is_null($entity_params['list_name'])) {
				// Return list object
				$items_list = new StdClass();
				$items_list->{$entity_params['list_name']} = $items;
				return $items_list;
			} else {
				return $items;
			}
		}
		// Cache miss
		return null;
	}

	// Record single object to database
	public function record_object($method, $id, $api_result) {
		if(!$api_result) {
			return false;
		}
		$entity = $this->_cached_methods[$method]["table"];
		$json_data = json_encode($api_result, JSON_HEX_QUOT);

		return $this->_db->prepared_exec(
			$this->_cached_parameters[$entity]["prep_insert"],
			array( $this->_db->format_string($id), $json_data )
		);
	}

	// Fetch API call results from database
	public function fetch_api_call($method, $args) {

		$encoded_args = json_encode($args);

		$result = $this->_db->prepared_fetch(
			$this->_prep_call_select,
			array( $method, $encoded_args )
		);
		if(!$result || !array_key_exists("result_ids", $result)) {
			return false;
		}
		return $result["result_ids"];
	}

	// Record API call
	public function record_api_call($method, $args, $id_array) {
		$encoded_args = json_encode($args);

		if( !is_array($id_array) || count($id_array) == 0) {
			return false;
		}

		for($i=0; $i<count($id_array); $i++){
			$id_array[$i] = $this->_db->format_string($id_array[$i]);
		}

		$id_string = implode(',',$id_array);

		return $this->_db->prepared_exec(
			$this->_prep_call_insert,
			array( $method, $encoded_args, $id_string )
		);
	}

	// Delete expired cahe items
	public function delete_expired($last_cache_sweep) {

		if( time() < $last_cache_sweep + $this->_cache_lifetime ) {
			// No need for cache sweep
			return false;
		}

		// Delete expired objects
		foreach($this->_cached_parameters as $entity => $params) {
			$table = $this->_table_prefix.$entity;
			$this->_del_expired($table);
		}

		// Delete expired API calls
		$table = $this->_table_prefix."cached_calls";
		$this->_del_expired($table);

		// Expired cache deleted
		return true;
	}

	// Purge cache
	public static function purge($db, $table_prefix) {
		$t = $table_prefix."cached_categories";
		$db->query("DELETE FROM {$t}");

		$t = $table_prefix."cached_courses";
		$db->query("DELETE FROM {$t}");

		$t = $table_prefix."cached_events";
		$db->query("DELETE FROM {$t}");

		$t = $table_prefix."cached_calls";
		$db->query("DELETE FROM {$t}");

		$t = $table_prefix."cached_tracks";
		$db->query("DELETE FROM {$t}");
	}

	// Delete expired items from single table
	private function _del_expired($table){
		$query = "
			DELETE FROM {$table}
			WHERE
				api_key = '{$this->_cache_key}'
				AND cached_at < DATE_SUB(NOW(), INTERVAL {$this->_cache_lifetime} SECOND)
		";
		$this->_db->query($query);
	}

	// Get SQL for constructing cache tables
	public static function get_table_structure($table_prefix)
	{
		$structure = array();

		$t = $table_prefix."cached_categories";
		$structure[$t] = "
		CREATE TABLE {$t} (
			wl_id int(11) NOT NULL AUTO_INCREMENT,
			id varchar(32) NOT NULL,
			value text NOT NULL,
			cached_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			api_key varchar(32) NOT NULL,
			PRIMARY KEY  (wl_id),
			KEY api_key (api_key),
			KEY id (id),
			KEY cached_at (cached_at)
		) DEFAULT CHARSET=utf8;";

		$t = $table_prefix."cached_courses";
		$structure[$t] = "
		CREATE TABLE {$t} (
			wl_id int(11) NOT NULL AUTO_INCREMENT,
			id varchar(32) NOT NULL,
			value text NOT NULL,
			cached_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			api_key varchar(32) NOT NULL,
			PRIMARY KEY  (wl_id),
			KEY api_key (api_key),
			KEY id (id),
			KEY cached_at (cached_at)
		) DEFAULT CHARSET=utf8;";

		$t = $table_prefix."cached_events";
		$structure[$t] = "
		CREATE TABLE {$t} (
			wl_id int(11) NOT NULL AUTO_INCREMENT,
			id varchar(32) NOT NULL,
			value text NOT NULL,
			cached_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			api_key varchar(32) NOT NULL,
			PRIMARY KEY  (wl_id),
			KEY api_key (api_key),
			KEY id (id),
			KEY cached_at (cached_at)
		) DEFAULT CHARSET=utf8;";

		$t = $table_prefix."cached_tracks";
		$structure[$t] = "
		CREATE TABLE {$t} (
			wl_id int(11) NOT NULL AUTO_INCREMENT,
			id varchar(32) NOT NULL,
			value text NOT NULL,
			cached_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			api_key varchar(32) NOT NULL,
			PRIMARY KEY  (wl_id),
			KEY api_key (api_key),
			KEY id (id),
			KEY cached_at (cached_at)
		) DEFAULT CHARSET=utf8;";

		$t = $table_prefix."cached_calls";
		$structure[$t] = "
		CREATE TABLE {$t} (
			id bigint(20) NOT NULL AUTO_INCREMENT,
			method varchar(32) NOT NULL,
			args text,
			result_ids text,
			cached_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			api_key varchar(32) NOT NULL,
			PRIMARY KEY  (id),
			KEY api_key (api_key)
		) DEFAULT CHARSET=utf8;";

		return $structure;
	}
}
