<?php
//  Stripe Payment Processor
//
require_once('lib/init.php');

class AdministrateWidgetCheckoutPaymentProcessorStripe extends AdministrateWidgetCheckoutPaymentProcessor {
	public function __construct($service, $widget, $order = false) {
		parent::__construct($service, $widget, $order);
		
		$this->display_name = "Stripe";
		$this->key = "stripe";
		$this->error_message = "";
		$this->stripe_client_secret = "";
		$this->success_json =  json_encode([
            'success' => true
        ]);


        if( function_exists("get_bloginfo") ) {
			// Wordpress configuration
			$this->stripe_public_key = $this->_get_option('public_key');
			$this->stripe_secret_key = $this->_get_option('secret_key');
		}
		else {
			// Weblink configuration
			$config = Administrate::api()->get_payment_config('stripe', false);
			
			if( is_object($config) ) {
				$this->stripe_public_key = $config->username;
				$this->stripe_secret_key = $config->password;
			}
			else {
				$this->stripe_public_key = null;
				$this->stripe_secret_key = null;
			}
		}
		
		if( empty($this->stripe_public_key) || empty($this->stripe_secret_key) ) {
			$this->is_configured = false;
		}
		else {
			$this->is_configured = true;
		}
	}
	
	// Perform card transaction
	public function card_transaction() {
		if(!$this->is_configured) {
			$this->error_message = "Payment processor not configured.";
			return false;
		}
		
		try {
			\Stripe\Stripe::setApiKey($this->stripe_secret_key);

			$order = $this->get_order();

			$stripe_data = array(
				// Amout in cents, currency and card token
				'currency'      => strtolower( $order->get_currency() ),
				'payment_method_data' => [
				    'type' => 'card',
                    'card' => ['token' => $this->get_token() ]
                ],

				// Stripe sends invoice to this e-mail,
				// if enabled in the user dashboard
				'receipt_email' => $order->get_buyer_email(),
                'confirmation_method' => 'automatic',
				'confirm' => 'true',
				// Metadata shows up in the user dashboard
				'metadata'       => array(
					'email'      =>	$order->get_buyer_email(),
					'company'    =>	$order->get_buyer_company(),
					'first_name' =>	$order->get_buyer_first_name(),
					'last_name'  =>	$order->get_buyer_last_name(),
					'phone'      =>	$order->get_buyer_phone(),
					'address1'   =>	$order->get_invoice_street(),
					'city'       =>	$order->get_invoice_city(),
					'territory'  =>	$order->get_invoice_territory(),
					'postal_code'=>	$order->get_invoice_postal_code(),
					'country'    =>	$order->get_invoice_country(),
				)
			);

			// NOTE: Prices must be *100, as stripe accepts the 'cent' amount.
			if($order->get_event_id()) {
				$mapped_items = $this->order->get_mapped_items_and_prices();
				$item_amount = 0;
				foreach($mapped_items as $item) {
					$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
				}
				$stripe_data['amount'] = (($this->order->get_event_gross_price() * $this->order->get_num_attendees()) + $item_amount) * 100;
				$stripe_data['description'] = $order->get_course_title();
			} else {
				$track = $this->order->get_track();
				list(
					$tax_amount,
					$tax_name,
					$net_price,
					$discounted_net_price,
					$gross_price,
					$discounted_gross_price,
					) = AdministrateLearningTrackWidget::price_detail(
					$track,
					$this->service->get_option('currency', 'pricing'),
					$this->order->get_num_attendees(),
					$this->get_order()->get_discount_type(),
					$this->get_order()->get_discount_rate()
				);

				$stripe_data['amount'] = $discounted_gross_price * 100;
				$stripe_data['description'] = $track->description;
			}

			$intent = \Stripe\PaymentIntent::create($stripe_data);
			return $this->generatePaymentResponse($intent);
		}
		catch (Exception $e) {
            return json_encode(['error' => 'Invalid PaymentIntent status']);
		}
	}

    function generatePaymentResponse($intent) {
        if ($intent->status == \Stripe\PaymentIntent::STATUS_REQUIRES_ACTION &&
            $intent->next_action->type == 'use_stripe_sdk') {
            # Tell the client to handle the action
            return json_encode([
                'requires_action' => true,
                'payment_intent_client_secret' => $intent->client_secret
            ]);
        } else if ($intent->status == 'succeeded') {
            # The payment didn’t need any additional actions and completed!
            # Handle post-payment fulfillment
            return $this->success_json;
        } else {
            $this->error_message = 'Invalid PaymentIntent Status';
            return json_encode(['error' => 'Invalid PaymentIntent status']);
        }
    }

    // Get card token
	public function get_token() {
		if( array_key_exists('stripe_token', $_POST) ) {
			return $_POST['stripe_token'];
		}
		return false;
	}
	
	// Get checkout order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$where = array(
				'order_id' => $_REQUEST['administrate_checkout_order_id']
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->service, $where);
		}
		return $this->order;
	}
	
	// Display template
	public function display_form() {
		require_once('checkout_stripe_form.php');
	}
}
