<?php
use PHPUnit\Framework\TestCase;
require_once('common/api/AdministrateCacheAlgorithm.php');
	
class AdministrateCacheAlgorithmTest extends TestCase{
	
	public function test_find_objects_empty() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array());
		$cache_mock->stage_table("table_2", array());
		$cache_mock->stage_table("calls", array());
	
		$cache_hit = $ca->find_objects("getSingleTest1", array( "id1" ));
		$this->assertSame( null, $cache_hit );
	}
	
	public function test_find_objects_single() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array("id1" => "Cached data"));
		$cache_mock->stage_table("table_2", array("id1" => "Other data"));
		$cache_mock->stage_table("calls", array());
		
		$cache_hit = $ca->find_objects("getSingleTest1", array( "id1" ));
		$this->assertEquals( "Cached data", $cache_hit );
		$cache_hit = $ca->find_objects("getSingleTest2", array( "id1" ));
		$this->assertEquals( "Other data", $cache_hit );
	}
		
	public function test_find_objects_single2() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array("id1" => "Cached data", "id2" => "Other data"));
		$cache_mock->stage_table("table_2", array());
		$cache_mock->stage_table("calls", array());
		
		$cache_hit = $ca->find_objects("getSingleTest1", array( "id2" ));
		$this->assertEquals( "Other data", $cache_hit );
	}
	
	public function test_find_objects_multiple_empty() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array());
		$cache_mock->stage_table("table_2", array());
		$cache_mock->stage_table("calls", array());
		
		$cache_hit = $ca->find_objects("getMultipleTest", array( "id2" ));
		$this->assertEquals( null, $cache_hit );
	}
	
	public function test_find_objects_multiple1() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array("id1" => "Cached data", "id2" => "Other data" ));
		$cache_mock->stage_table("table_2", array());
		$cache_key = json_encode(array("getMultipleTest",array("arg1"=>10)));
		$cache_mock->stage_table("calls", array( $cache_key => "id1" ));
		
		$cache_hit = $ca->find_objects("getMultipleTest", array( "arg1" => 10 ));
		$this->assertEquals( array("Cached data"), $cache_hit );
	}
	
	public function test_find_objects_multiple2() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array("id1" => "Cached data", "id2" => "Other data", "id3" => "More data" ));
		$cache_mock->stage_table("table_2", array());
		$cache_key = json_encode(array("getMultipleTest",array("arg1"=>10)));
		$cache_mock->stage_table("calls", array( $cache_key => "id1, id3" ));
		
		$cache_hit = $ca->find_objects("getMultipleTest", array( "arg1" => 10 ));
		$this->assertEquals( array("Cached data", "More data"), $cache_hit );
	}
	
	public function test_record_objects() {
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock = new AdministrateCache_TestMock();
		$ca = new AdministrateCacheAlgorithm($cache_mock);
		
		$cache_mock->stage_table("table_1", array());
		$cache_mock->stage_table("table_2", array());
		$cache_mock->stage_table("calls", array());
		
		$obj1 = new stdClass();
		$obj1->id = "id1";
		$obj1->data = "Test data";
		
		$obj2 = new stdClass();
		$obj2->id = "id2";
		$obj2->data = "Other data";
		
		$obj_list = new stdClass();
		$obj_list->list = array($obj1,$obj2);
		
		$call = "getMultipleTest";
		$args = array( "arg1"=>10 );
		
		$ca->record_objects($call, $args, $obj_list);
		$expected_cache = array(json_encode(array($call,$args)) => array("id1","id2"));
		
		$this->assertEquals(array("id1"=>$obj1,"id2"=>$obj2),$cache_mock->get_table('table_1'));
		$this->assertEquals(array(),$cache_mock->get_table('table_2'));
		$this->assertEquals($expected_cache, $cache_mock->get_table('calls'));
	}
}

class AdministrateCache_TestMock {
	private $_objects = null;
	private $_cached_methods = array(
		'getSingleTest1' => array("table" => 'table_1', 'multiple' => false),
		'getSingleTest2' => array("table" => 'table_2', 'multiple' => false),
		'getMultipleTest' => array("table" => 'table_1', 'multiple' => true),
	);
	
	public function stage_table($table, $entries) {
		$this->_objects[$table] = $entries;
	}
	public function get_table($table) {
		return $this->_objects[$table];
	}
	
	public function is_call_cacheable($method) {
		return array_key_exists($method, $this->_cached_methods);
	}
	
	public function is_result_multiple($method) {
		return $this->_cached_methods[$method]['multiple'];	
	}
	
	public function get_call_type($method)  {
		return $this->_cached_methods[$method]["table"];
	}
	
	public function get_call_parameters($call_type)  {
		return array('list_name' => 'list', 'id_column' => 'id');
	}
	
	public function fetch_single_object($method, $id, $entity_params) {
		$call_type = $this->get_call_type($method);
		$t = $this->get_table($call_type);
		
		if( array_key_exists($id, $t) ){
			return $t[$id];
		}
		return null;
	}
	
	public function fetch_multiple_objects($method, $id_array, $entity_params) {
		$call_type = $this->get_call_type($method);
		$t = $this->get_table($call_type);
		$ids = explode(", ",$id_array);
		
		$result = array();
		foreach($ids as $id) {
			if( array_key_exists($id, $t) ){
				$result[] = $t[$id];
			}
		}
		return $result;
	}
	
	public function fetch_api_call($method, $args, $entity_params) {
		$id = json_encode(array($method,$args));
		$t = $this->get_table("calls");
		
		if( array_key_exists($id, $t) ){
			return $t[$id];
		}
		return null;
	}
	
	public function record_object($method, $id, $api_result) {
		$call_type = $this->get_call_type($method);
		$t = $this->get_table($call_type);
		$t[$id] = $api_result;
		$this->stage_table($call_type, $t);
	}
	
	public function record_api_call($method, $args, $id_array) {
		$id = json_encode(array($method,$args));
		$t = $this->get_table("calls");
		
		$t[$id] = $id_array;
		$this->stage_table("calls", $t);
	}
}
