<?php
//  Plugin admin interface
//
class AdministratePluginAdmin {
	public function __construct($plugin) {
		$this->plugin = $plugin;
		
		// Wordpress actions
		add_action('admin_menu',    array($this, 'initialize_menu'));
		add_action('admin_init',    array($this, 'initialize_page'));
	}
		
	public function initialize_menu() {
		if(!current_user_can('activate_plugins')) {
			return;
		}
		
		wp_enqueue_style($this->plugin->add_namespace('common'), plugin_dir_url(__FILE__).'resources/common.css');
		wp_enqueue_style($this->plugin->add_namespace('admin'), plugin_dir_url(__FILE__).'resources/admin.css');
		
		// Add root menu
		$display_function = array($this, 'display_page');
		$page_id          = $this->plugin->add_namespace('home');
		$menu_title       = $this->plugin->get_config("plugin", "menu_title");
		$icon             = plugin_dir_url(__FILE__)."resources/images/administrate-20px-logo.png";
		
		add_menu_page($menu_title, $menu_title, 'manage_options', $page_id, $display_function, $icon, 3);
		
		$this->tabs = array();
		$this->tabs[$page_id] = __("Welcome");
		
		// Add tab and menu for each options group
		foreach ($this->plugin->get_options() as $group) {
			$page_sub_id = $group['key'];
			$this->tabs[$page_sub_id] = $group['title'];
			add_submenu_page($page_id, $group['title'], $group['title'], 'manage_options', $page_sub_id, $display_function);
		}
			
		// Add tab and menu for static page templates
		foreach ($this->plugin->get_config("plugin", "admin_pages") as $page => $title) {
			$page_sub_id = $this->plugin->add_namespace($page);
			$this->tabs[$page_sub_id] = $title;
			add_submenu_page($page_id, $title, $title, 'read', $page_sub_id, $display_function);	
		}
	}
	
	public function initialize_page() {
		$this->_register_settings();
		
		if( $this->plugin->is_admin_page() ) {
			
			$js_dir = plugin_dir_url(__FILE__).'/js';
			$dependencies = array('jquery', 'jquery-ui-core', 'jquery-ui-widget');
			wp_enqueue_script('administrate_table_sorter', $js_dir.'/jquery.tablesorter.js', $dependencies, false, true);
			
			wp_enqueue_script('administrate_log_table', $js_dir.'/jquery.administrate_log_table.js', $dependencies, false, true);
			
			wp_enqueue_script('administrate_admin_monitoring', $js_dir.'/admin-monitoring.js', $dependencies, false, true);
			
			wp_enqueue_script('administrate_admin', $js_dir.'/admin.js', $dependencies, false, true);

			$this->_dispatch_commands();
		}
	}
	
	// Register Wordpress settings
	private function _register_settings() {
		$display_function  = array($this, 'display_field');
		$sanitize_funciton = array($this, 'sanitize');
		
		foreach( $this->plugin->get_options() as $group_key => $group ) {
			
			$wp_options_key = $group['key'];
			$wp_options = get_option($wp_options_key);
			
			add_settings_section($wp_options_key, null, null, $wp_options_key);
			
			foreach( $group['fields'] as $field_key => $field ) {
					
				if( array_key_exists('display', $field) && !$field['display'] ) {
					continue;
				}
				
				if( is_array($wp_options) && array_key_exists($field_key, $wp_options) ) {
					// There is a saved WP option value
					$field['value'] = $wp_options[$field_key];
				}
				
				if( !array_key_exists('required', $field) ) {
					$field['required'] = false;
				}
				
				$field['optionsKey'] = $wp_options_key;
				$field['groupKey'] = $group_key;
				$field['key'] = $field_key;
				
				add_settings_field($field_key, $field['label'], $display_function, $wp_options_key, $wp_options_key, $field);
				
			}
			// Register settings group
			register_setting($wp_options_key, $wp_options_key, $sanitize_funciton);
		}
	}
	
	public function sanitize($input) {
		$wp_options_key = $_POST['option_page'];
		$wp_options = get_option($wp_options_key);
		
		$lprefix = strlen($this->plugin->add_namespace(''));
		$group_id = substr($wp_options_key, $lprefix, strlen($wp_options_key) - $lprefix - strlen('_options'));
		$output = array();
		
		foreach( $input as $option_id => $value ) {
			$field_details = $this->plugin->get_option_field($option_id, $group_id);
			$error = '';
			
			if( !isset($field_details) ) {
				continue;
			}
			if( $this->_validate_field($value, $field_details, $error) ) {
				$output[$option_id] = $this->_sanitize_field($value, $field_details);
			}
			else {
				add_settings_error($option_id, $option_id . $this->plugin->get_key_delimiter() . 'required', $field_details['label'] . __(' field is required.', 'administrate'));
				$output[$option_id] = $wp_options[$option_id];
			}
		}
		return $output;
	}
	
	private function _validate_field($value, $field, &$error) {
		if( !is_array($field) ) {
			return true;
		}
		
		if( is_array($value) ) {
			foreach( $value as $sub_id => $sub_value ) {
				if( !$this->_validate_field($sub_value, $field, $error) ) {
					return false;
				}
			}
			return true;
		}
		
		if( empty($value) ) {
			if( array_key_exists('required', $field) && $field['required'] ) {
				$error = $field['label'] . __(' field is required.', 'administrate');
				return false;
			}
			return true;
		}
		
		if( ($field['type'] == 'email') ) {
			if( !is_email($value) ) {
				$error = $field['label'] . __(' field must be a valid email address.', 'administrate');
				return false;
			}
			return true;
		}
		
		if( array_key_exists('default', $field) && is_numeric($field['default']) ) {
			if( !is_numeric($value) ) {
				$error = $field['label'] . __(' field must be a number.', 'administrate');
				return false;
			}
			return true;
		}
		
		return true;
	}
	
	private function _sanitize_field($value, $field) {
		if( is_array($value) ) {
			foreach( $value as $sub_id => $sub_value ) {
				$value[$sub_id] = $this->_sanitize_field($sub_value, $field);
			}
			return serialize($value);
		}
		
		if( $field['type'] == 'checkbox' ) {
			return intval($value) ? 1 : 0;
		}
		
		if( array_key_exists('default', $field) && is_numeric($field['default']) ) {
			return intval($value);
		}
		
		if( !array_key_exists('allow_html', $field) || (array_key_exists('allow_html', $field) && !$field['allow_html']) ) {
			return wp_strip_all_tags( wp_check_invalid_utf8($value,true) );
		}
		return $value;
	}
	
	// Display settings page
	public function display_page() {
		require_once($this->plugin->get_path('/views/admin/page.php'));
	}
	
	// Display settings field
	public function display_field($field) {
		if( array_key_exists('display', $field) && !$field['display'] ) {
			return;
		}
		
		if( array_key_exists($field['key'], $_POST) ) {
			$field['value'] = $_POST[ $field['key'] ];
		}
		
		if ($field['type'] == 'custom') {
			$template_name = $field['key'];
		}
		else {
			$template_name = $field['type'];
			
			if( in_array($template_name, array('email', 'url', 'password')) ) {
				$template_name = 'text';
			}
		}
		$template = $this->plugin->get_path('/views/admin/option_field_'.$template_name.'.php');
		
		if (file_exists($template)) {
			include($template);	
		}
	}
	
	// Dispatch user commands
	private function _dispatch_commands() {
		if (isset($_POST['purge_cache']) || isset($_GET['settings-updated'])) {
			$this->plugin->purge_cache();
		}
		else if (isset($_POST['build_cache'])) {
			$this->plugin->build_cache();
			$this->plugin->show_notice(__('You have successfully built the cache.', 'administrate'));
		}
		else if (isset($_POST['refresh_urls'])) {
			$this->plugin->refresh_urls();
		}
		else if (isset($_POST['save_urls'])) {
			$this->plugin->save_urls($_POST);
		}
	}
}
