<?php
// Administrate Wordpress plugin monitoring and health-check
//
class AdministrateWordpressMonitor {
	public function __construct($plugin, $db) {
		$this->plugin = $plugin;
		$this->db = $db;
		$this->live_monitor = intval( $this->plugin->get_option('live_monitoring', 'api') );
		$this->update_url = $plugin->get_update_url();
		$this->docs_url = $plugin->get_docs_url();

		// Minimal and optimal requirements
		$this->requirements_min = $this->plugin->get_config('plugin', 'requirements_min');
		$this->requirements_opt = $this->plugin->get_config('plugin', 'requirements_opt');

		if( is_admin() ) {
			add_action('wp_loaded', array($this, 'report_json'));
		}
		else {
			add_action('wp_footer', array($this, 'record_api_log'));
		}
	}

	// Generate report and return back JSON with summary
	public function report_json() {
		wp_enqueue_style($this->plugin->add_namespace('admin'), plugin_dir_url(__FILE__).'resources/admin.css');

		if( !isset($_REQUEST['administrate_monitoring_ajax']) ) {
			return;
		}
		$this->report_generate();
		$this->report_save();

		wp_send_json( json_encode($this->_report_summary) );
	}

	// Send e-mail with attached report file
	public function report_send($user_email, $user_message) {
		$site_url   = get_site_url();
		$to         = $this->plugin->get_config('plugin', 'support_email');
		$subject    = "Wordpress Monitoring Report [{$site_url}]";
		$message    = "Website: {$site_url}\n";
		$message   .= "Contact e-mail: {$user_email}\n";
		$message   .= "User message: \n". ( empty($user_message) ? "-" : $user_message );
		$headers    = "";
		$attachment = $this->report_directory().$this->report_filename();

		return wp_mail( $to, $subject, $message, $headers, $attachment );
	}

	// Save report file
	public function report_save() {
		if( !is_writable( $this->report_directory() ) ) {
			return false;
		}

		$file_path = $this->report_directory() . $this->report_filename();

		if( !($file=fopen($file_path, "w")) ) {
			return false;
		}
		if( !fwrite($file, $this->_report_content) ) {
			return false;
		}
		fclose($file);
		return true;
	}

	public function report_directory() {
		$upload_dir = wp_upload_dir();
		return trailingslashit($upload_dir['path']);
	}

	public function report_url() {
		$upload_dir = wp_upload_dir();
		return trailingslashit($upload_dir['url']);
	}

	public function report_filename() {
		$date = date("Y_M_d");
		return "Administrate_Report_".$date.".txt";
	}

	// Generate report content
	public function report_generate() {
		$this->_report_content = "Worpdress report generated on ".date("Y-M-d")."\n\n";
		$this->_report_content .= "Site URL: ".get_site_url()."\n\n";

		$this->_report_current_section = "";

		$this->_report_summary = array(
			"file_url"  => $this->report_directory().$this->report_filename(),
			"file_name" => $this->report_filename(),
			"sections"  => array( )
		);

		$this->_r_server_config();
		if( $this->has_curl() && $this->has_soap() )
		{
			$this->_r_bandwidth_test();
			$this->_r_api_test();
			$this->_r_db_test();
			$this->_r_error_log();
		}
		$this->_r_plugin_config();
	}

	// Generate report (Server Config)
	private function _r_server_config() {
		global $wpdb, $wp_scripts;

		$min_fail = array();
		$opt_fail = array();

		$this->_r_section_header("Server Configuration Details");

		// PHP version
		//
		$phpver = phpversion();
		$this->_r_var("PHP version", $phpver);
		$this->_r_var("PHP_OS", PHP_OS);

		if( version_compare($phpver, $this->requirements_min['php']['ver'], '<') ) {
			$min_fail[] = "PHP";
		}
		else if( version_compare($phpver, $this->requirements_opt['php']['ver'], '<') ) {
			$opt_fail[] = "PHP";
		}

		// Curl extention
		//
		if( !$this->has_curl() ) {
			$min_fail[] = "Curl extension";
		}

		// Soap extention
		//
		if( !$this->has_soap() ) {
			$min_fail[] = "SOAP extension";
		}

		// Server software
		//
		if( array_key_exists("SERVER_SOFTWARE",$_SERVER) ) {
			$this->_r_var("SERVER_SOFTWARE", $_SERVER["SERVER_SOFTWARE"]);
		}

		if( array_key_exists("SERVER_ADMIN",$_SERVER) ) {
			$this->_r_var("SERVER_ADMIN", $_SERVER["SERVER_ADMIN"]);
		}

		// Database version
		//
		$db_version = $wpdb->get_var("SELECT VERSION() AS version");

		if( $db_version ) {
			$this->_r_var("DB Version", $db_version);

			if( version_compare($db_version, $this->requirements_min['mysql']['ver'], '<') ) {
				$min_fail[] = "MySQL";
			}
			else if( version_compare($db_version, $this->requirements_opt['mysql']['ver'], '<') ) {
				$opt_fail[] = "MySQL";
			}
		}

		// jQuery version
		//
		wp_enqueue_script('jquery');
		if( array_key_exists("jquery", $wp_scripts->registered) ) {
			$jquery_version = $wp_scripts->registered['jquery']->ver;
			$this->_r_var("jQuery Version", $jquery_version);

			if( version_compare($jquery_version, $this->requirements_min['jquery']['ver'], '<') ) {
				$min_fail[] = "jQuery";
			}
			else if( version_compare($jquery_version, $this->requirements_opt['jquery']['ver'], '<') ) {
				$opt_fail[] = "jQuery";
			}
		}

		// Min configuration
		//
		$notes = array();
		$note_str = __( 'Your web server does not support the minimal requirement for ');

		for( $i=0; $i < count($min_fail); $i++ ) {
			$note_str = $note_str . $min_fail[$i];
			if( $i == count($min_fail) - 2 ) {
				$note_str = $note_str . " and ";
			}
			else if( $i < count($min_fail) - 1 ) {
				$note_str = $note_str . ", ";
			}
		}
		$note_str = $note_str . ".";

		if( count($min_fail) ) {
			$notes[] = $note_str;
		}

		// Optimal configuration
		//
		$note_str = __( 'Your version of ');

		for( $i=0; $i < count($opt_fail); $i++ ) {
			$note_str = $note_str . $opt_fail[$i];
			if( $i == count($opt_fail) - 2 ) {
				$note_str = $note_str . " and ";
			}
			else if( $i < count($opt_fail) - 1 ) {
				$note_str = $note_str . ", ";
			}
		}
		$note_str = $note_str . __( ' is compatible but not up-to-date.');

		if( count($opt_fail) ) {
			$notes[] = $note_str;
		}

		// Summary
		//
		if( empty($min_fail) && empty($opt_fail) ) {
			$notes[] = __("Your server is running up-to-date versions of the required software.");
		}
		else {
			$help_str = __("Please visit the ")."<a target='_blank' href='{$this->docs_url}'>".__("reference page")."</a>.";
			$notes[0] = $notes[0] . " " . $help_str;
		}
		$this->_r_section_summary( empty($min_fail) ? ( empty($opt_fail) ? 100 : 50 ) : 0, $notes);
	}

	private function has_curl() {
		return function_exists('curl_version');
	}

	private function has_soap() {
		return class_exists('SoapClient');

	}

	// Generate report (API Test)
	private function _r_api_test() {
		$notes = array();
		$pass = $opt_pass = true;
		$this->_r_section_header("API Connection Test");

		$this->plugin->disable_cache();
		Administrate::log()->clear(E_ALL);

		$t_start = microtime(true);
		$this->plugin->build_cache();
		$t_end = microtime(true);
		$dt = $t_end - $t_start;

		$log = Administrate::log()->get(E_ALL);

		$n_error   = count(Administrate::log()->get(E_ERROR));
		$n_warning = count(Administrate::log()->get(E_WARNING));
		$n_soap    = array_reduce($log, array('self', 'count_soap_calls'));
		$n_rest    = array_reduce($log, array('self', 'count_rest_calls'));
		$t_worst   = array_reduce($log, array('self', 'biggest_exec_time'));
		$t_avg     = $dt / max($n_soap+$n_rest,1);

		if( $n_error ) {
			$notes[] = __("An error has occurred during the API test. Please make sure you have read <a href='{$this->docs_url}' target='_blank'>how to setup your API connection.</a>");
			$pass = false;
		}

		if( $n_warning ) {
			$notes[] = __("Some non-critical warnings have occurred during your test. Please send us your report.");
			$opt_pass = false;
		}

		$this->_r_var("Errors", ($n_error) );
		$this->_r_var("Warnings", ($n_warning) );
		$this->_r_var("SOAP Calls", $n_soap );
		$this->_r_var("REST Calls", $n_rest );
		$this->_r_var("Total execution time", number_format($dt,2) . "sec" );
		$this->_r_var("Worst call time", number_format($t_worst,2) . "sec" );
		$this->_r_var("Avarage call time", number_format($t_avg,2) . "sec" );

		$this->_r_dump_log();

		// Summary
		//
		if( empty($notes) ) {
			$notes[] = "Your API connection to Administrate was successful. No errors or warnings were detected.";
		}
		$this->_r_section_summary($pass ? ( $opt_pass ? 100 : 50 ) : 0, $notes);
	}

	public static function count_soap_calls($carry, $log_message) {
		return $carry + ( strstr($log_message, "SOAP call") ? 1 : 0 );
	}

	public static function count_rest_calls($carry, $log_message) {
		return $carry + ( strstr($log_message, "REST call") ? 1 : 0 );
	}

	public static function biggest_exec_time($carry, $log_message) {
		if( strstr($log_message, "exec_time=") ) {
			if( preg_match('/\d+\.\d+/', $log_message, $matches) ) {
				$exec_time = floatval($matches[0]);

				if( $exec_time > $carry ) {
					return $exec_time;
				}
			}
		}
		return $carry;
	}

	// Generate report (DB Test)
	private function _r_db_test() {
		$notes = array();
		$pass = true;
		$this->_r_section_header("Cache Test");

		$cache_time = intval( $this->plugin->get_option('cache_timeout', 'api') );

		if( !$cache_time ) {
			$tab_link = "?page=administrate_api_options";
			$notes[0] = __("Caching is disabled by the user.\n");
			$this->_report_content .= $notes[0];
			$notes[0] .= __("You can enable it in the <a href='{$tab_link}'>API Settings</a> tab.\n");
			$this->_r_section_summary(50, $notes);
			return;
		}

		$this->plugin->enable_cache();
		$this->plugin->build_cache();
		Administrate::log()->clear(E_ALL);

		$t_start = microtime(true);
		$this->plugin->build_cache();
		$t_end = microtime(true);
		$dt = $t_end - $t_start;

		$log = Administrate::log()->get(E_ALL);

		$n_error    = count(Administrate::log()->get(E_ERROR));
		$n_warning  = count(Administrate::log()->get(E_WARNING));
		$t_db_total = array_reduce($log, array('self', 'count_db'));
		$n_hit      = array_reduce($log, array('self', 'count_hit'));
		$n_miss     = array_reduce($log, array('self', 'count_miss'));
		$t_avg      = $dt / max($n_hit + $n_miss,1);

		$this->_r_var("Cache timeout", $cache_time."sec");

		$this->_r_var("Total db queries", $t_db_total);
		$this->_r_var("Cache hits", $n_hit);
		$this->_r_var("Cache misses", $n_miss);

		$str_total = number_format($dt,2)." sec";
		$str_avg   = number_format($t_avg,3)." sec";

		$this->_r_var("Total retrieval time", $str_total);
		$this->_r_var("Average retrieval time", $str_avg);

		$this->_r_dump_log();

		if( !$n_hit ) {
			$notes[] = __("Caching does not appear to be active, please send us your report.");
			$pass = false;
		}

		// Summary
		//
		if( empty($notes) ) {
			$notes[] = __("Your cache is operating normally.");
			$notes[] = __("Cache hits {$n_hit}, Cache misses {$n_miss}, Average retrieval time {$str_avg}.");
		}
		else {
			$help_str = __("Please visit the ") ."<a target='_blank' href='{$this->docs_url}'>".__("reference page")."</a>.";
			$notes[0] = $notes[0] . " " . $help_str;
		}
		$this->_r_section_summary($pass ? 100 : 0, $notes);
	}

	public static function count_db($carry, $log_message) {
		return $carry + ( strstr($log_message, "DB:") ? 1 : 0 );
	}
	public static function count_hit($carry, $log_message) {
		return $carry + ( strstr($log_message, "Cache hit") ? 1 : 0 );
	}
	public static function count_miss($carry, $log_message) {
		return $carry + ( strstr($log_message, "Cache miss") ? 1 : 0 );
	}

	// Generate report (Bandwidth Test)
	private function _r_bandwidth_test() {
		$notes = array();
		$pass = $opt_pass = true;

		$content = $this->_r_section_header("Server Bandwidth Test");

		if( !function_exists('curl_version') ) {
			$notes[0] = __("Curl extension is not available, aborting test.");
			$this->_report_content .= $notes[0]."\n";
			$this->_r_section_summary(0, $notes);
			return;
		}

		$dt = 0;
		$size = 0;
		$n = 0;

		$url = parse_url( Administrate::api()->get_credentials()->get_soap_url() );

		$resources = array(
			"/css/bootstrap/bootstrap.css",
			"/images/system-customFavicon.ico",
		);

		// Fetching a small static resource many times to simulate an API call
		// This is a more accurate measure than than fetching one big file once
		foreach($resources as $resource_url) for($i=0; $i<20; $i++) {
			$ch = curl_init($url["scheme"]."://".$url["host"] .$resource_url."?t=".time().$i);
			curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array("Cache-Control: no-cache", "Pragma: no-cache"));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
			curl_setopt($ch, CURLOPT_TIMEOUT, 2);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

			$t_start = microtime(true);
			if ( !curl_exec($ch) ) {
				continue;
			}
			$t_end = microtime(true);

			$dt += $t_end - $t_start;
			$size += curl_getinfo($ch, CURLINFO_SIZE_DOWNLOAD);

			curl_close($ch);
			$n++;
		}

		if( $n == 0 ) {
			$notes[0] = __("There was a problem establishing a connection to the network resource.");
			$this->_r_section_summary(0, $notes);
			return;
		}

		$size_MB = $size/(1024*1024);	// in MB
		$size_Mb = $size_MB*8;			// in Mbit
		$bandwidth = $size_Mb / $dt;	// im Mbit/s

		if( $dt == 0 ) {
			$notes[0] = __("There was a problem establishing a connection to the network resource.");
			$this->_r_section_summary(0, $notes);
			return;
		}

		$str_bandwidth = number_format($bandwidth,2);

		$this->_r_var("Total downloaded:", number_format($size_MB,2)."MB, {$n} simulated calls." );
		$this->_r_var("Total time", number_format($dt,2)."sec" );
		$this->_r_var("Effective call speed", $str_bandwidth."Mbit/s" );

		// Summary
		//
		if( $bandwidth >= 1 ) {
			$str_connection = "excellent";
		}
		else if( $bandwidth >= 0.05 ) {
			$str_connection = "adequate";
		}
		else {
			$str_connection = "poor";
			$opt_pass = false;
		}

		$notes[] = __("Estimated connection speed is {$str_connection}. Effective call speed {$str_bandwidth}Mbit/s, {$n} simulated calls.");
		$this->_r_section_summary($pass? ( $opt_pass? 100 : 50 ) : 0, $notes);
	}

	// Generate report (Live Error Log)
	private function _r_error_log() {
		$notes = array();
		$this->_r_section_header("Live Error Log");

		if( !$this->live_monitor ) {
			$tab_link = "?page=administrate_api_options";
			$notes[0] = __("Live monitoring is deactivated by the user.\n");
			$this->_report_content .= $notes[0];
			$notes[0] .= __("You can activate it in the <a href='{$tab_link}'>API Settings</a> tab.\n");
			$this->_r_section_summary(50, $notes);
			return;
		}

		$error_logs = $this->db->query_fetch("SELECT log, log_date FROM {$this->db->get_table_prefix()}administrate_live_monitor");

		if( $this->db->get_last_num_records() == 1) {
			// The database unwraps result sets consisting of one item
			$result = array($result);
		}

		if( !count($error_logs) ) {
			$notes[0] = __("No errors or warnings were found in the event log.");
			$this->_report_content .= $notes[0]."\n";
			$this->_r_section_summary(100, $notes);
			return;
		}

		foreach( $error_logs as $log ) {
			$log_messages = unserialize( $log["log"] );

			if( count($log_messages) == 0) {
				continue;
			}

			foreach( $log_messages as $message ) {
				$this->_report_content .= $message. "\n";
			}
			$this->_report_content .= "---\n";
		}

		$notes[0] = __("There are some warnings or errors in the event log, please submit your report.");
		$this->_r_section_summary(0, $notes);
	}

	// Live event log
	// This function should be registered in wp_footer
	public function record_api_log() {
		if( !$this->live_monitor ) {
			return;
		}

		$log = Administrate::log()->get(E_ERROR|E_WARNING);

		if( count($log) == 0 ) {
			return;
		}

		$pstmt = $this->db->prepare("INSERT INTO {$this->db->get_table_prefix()}administrate_live_monitor (log) VALUES ( (?) )", "s");
		$this->db->prepared_exec($pstmt, serialize( $log ) );

		// Clean up
		//
		$last_log_sweep = get_option('administrate_event_log_sweep', time());
		$log_lifetime   = $this->live_monitor * 24 * 60 * 60;

		if( time() >= $last_log_sweep + $log_lifetime ) {
			update_option('administrate_event_log_sweep', time() );
			$this->db->query("DELETE FROM {$this->db->get_table_prefix()}administrate_live_monitor");
		}
	}

	// Generate report (Wordpress and Plugin Config)
	private function _r_plugin_config() {
		$notes = array();
		$pass = $opt_pass = true;
		$this->_r_section_header("Wordpress and Plugin Version");

		// Plugin version
		//
		$plugin_headers = get_plugin_data(  $this->plugin->get_config('plugin', 'file') );
		$this->_r_var("Installed plugin version", $plugin_headers["Version"] );

		if( !function_exists('curl_version') ) {
			$notes[0] = __("Curl extension is not available, aborting test.");
			$this->_report_content .= $notes[0]."\n";
			$this->_r_section_summary(0, $notes);
			return;
		}

		$ch = curl_init( $this->update_url );
		curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array("Cache-Control: no-cache", "Pragma: no-cache" ));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
		curl_setopt($ch, CURLOPT_TIMEOUT, 8);

		if ( $update_str = curl_exec($ch) ) {
			$update_json = json_decode($update_str);

			$plugin_ver = $plugin_headers["Version"];

			if( version_compare($plugin_ver, $update_json->version, '<') ) {
				$notes[] = __("Your version of the Administrate plugin '{$plugin_ver}' is not up-to-date. Please update to '{$update_json->version}'.");
				$pass = false;
			}
		}
		curl_close($ch);

		// Wordpress version
		//
		$wp_ver = get_bloginfo("version", "display");
		$wp_ver_latest = $this->_latest_wp_ver();

		$this->_r_var("Wordpress version", $wp_ver );

		if( version_compare($wp_ver, $this->requirements_min['wordpress']['ver'], '<') ) {
			$notes[] = __('Your web server does not support the minimal requirement for Wordpress.');
			$pass    = false;
		}
		else if( version_compare($wp_ver, $wp_ver_latest, '<') ) {
			$notes[] = __("Your version of Wordpress '{$wp_ver}' is compatible but not up-to-date.");
			$opt_pass = false;
		}

		// Wordpress settings
		//
		$wp_options = wp_load_alloptions();
		$exclude = array(
			"wp_user_roles", "ftp_credentials", "cron",
			"password", "signature", "key", "md5",
			"auth_", "mailserver_", "_transient", "_site_transient"
		);

		$wp_filtered = $this->_filter_config($wp_options,$exclude);

		// Generate report setion
		foreach($wp_filtered as $name=>$value) {
			$this->_report_content .= $this->_r_var($name, $value);
		}

		// Summary
		//
		if( empty($notes) ) {
			$notes[] = __("You are running an up-to-date version of Wordpress and the Administrate plugin.");
		}
		else {
			$help_str = __("Visit the ") ."<a target='_blank' href='{$this->docs_url}'>".__("reference page")."</a>.";
			$notes[0] = $notes[0] . " " . $help_str;
		}
		$this->_r_section_summary($pass ? ( $opt_pass ? 100 : 50 ) : 0, $notes);
	}

	// Latest Wordpress version
	private function _latest_wp_ver() {
		$default_ver = $this->requirements_opt['wordpress']['ver'];
		$version_service = "https://api.wordpress.org/core/version-check/1.7/";

		if ( $update = wp_remote_post($version_service) ) {

			if( !array_key_exists("body",$update) ) {
				return $default_ver;
			}

			$update_json = json_decode($update["body"]);

			if( !is_object($update_json) ) {
				return $default_ver;
			}

			if( !property_exists($update_json, "offers") ) {
				return $default_ver;
			}

			if( !count($update_json->offers) ) {
				return $default_ver;
			}

			$wp_offer = $update_json->offers[0];

			if( !property_exists($wp_offer, "version") ) {
				return $default_ver;
			}

			if( version_compare($wp_offer->version, $default_ver, '<') ) {
				return $default_ver;
			}

			// Return latest version
			return $wp_offer->version;
		}

		// Default to static parameter if update service is unavailable
		return $default_ver;
	}

	private function _filter_config($options, $exclude) {
		$filtered = $options;

		foreach($options as $name => $value) {
			if( is_array($value) ) {
				if( empty($value) ) {
					$filtered[$name] = "";
				}
				else {
					$filtered[$name] = $this->_filter_config($value,$exclude);
				}
			}

			foreach($exclude as $exclude_name) {
				if( strstr($name,$exclude_name) ) {
					unset( $filtered[$name] );
				}
			}
		}
		return $filtered;
	}

	// Dump event log errors and warnings
	private function _r_dump_log() {
		$error_log = Administrate::log()->get(E_ERROR|E_WARNING);

		if( count($error_log) ) {

			$this->_report_content .= "\n";
			$this->_report_content .= "Some errors or warnings have occured during the test, ";
			$this->_report_content .= "please examine the event log:\n\n";

			foreach($error_log as $log_message) {
				$this->_report_content .= $log_message . "\n";
			}
		}
	}

	// Output section header
	private function _r_section_header($title) {
		$separator = "\n" . str_repeat( "=",strlen($title) ) . "\n";
		$this->_report_content .= $separator . $title . $separator;
		$this->_current_report_section = $title;
	}

	// Output section summary
	public function _r_section_summary($grade, $notes = array()) {
		$this->_report_summary["sections"][ $this->_current_report_section ] = array(
			"grade" => round($grade),
			"notes"  => $notes
		);
	}

	// Output report variable
	public function _r_var($title, $value) {
		$this->_report_content .= $title." = ".var_export($value,true)."\n";
	}
}
