<?php
//  Include dependencies
$path = dirname(__FILE__);
require_once($path.'/WPPluginPatternWidget.php');
require_once($path.'/WPPluginPatternDAO.php');
require_once($path.'/WPPluginUrlAndPathResolver.php');

//  WPPluginPattern
abstract class WPPluginPattern extends Administrate {

	//  Properties
	protected $namespace = "myplugin";
	protected $keyDelimiter = "_";
	public $db = false;
	protected $path = false;
	protected $url = false;
	public $debug = false;
	protected $config = array();
	protected $options = array();
	protected $optionValues = array();
	protected $transferableSubfieldProperties = array(
		'required',
		'max_length',
		'default',
		'hint'
	);
	protected $tableMaps = array();
	protected $tables = array();
	protected $jsEnabled = false;
	protected $controller;
	protected $plugin;
	protected $log_types = array(
		'error'		=>	E_USER_ERROR,
		'warning'	=>	E_USER_WARNING,
		'notice'	=>	E_USER_NOTICE
	);
	protected $last_log_time = 0;
	protected $messages = array();

	//  Constructor
	public function __construct($config, &$db, $debug = false) {

		//  Register activation / deactivation hooks
		register_activation_hook(__FILE__, array($this, 'activate'));
		register_deactivation_hook(__FILE__, array($this, 'deactivate'));

		//  Save all the config
		$this->config = $config;

		//  Save handle to the database
		$this->db = &$db;

		//  Save the debug flag
		$this->debug = $debug;

		//  Save the namespace
		if (array_key_exists('plugin', $this->config) && array_key_exists('namespace', $this->config['plugin'])) {
			$this->namespace = $this->config['plugin']['namespace'];
		}

		//  Set table names
		foreach ($this->tables as $key=>$fields) {
			$this->tableMaps[$key] = $this->db->prefix . $this->add_namespace($key);
		}

		$pathAndUrlResolver = new WPPluginUrlAndPathResolver(get_class($this), $this->config);
		$this->path = $pathAndUrlResolver->get_path();
		$this->url = $pathAndUrlResolver->get_url();

		//  Initialize plugin translations
		load_plugin_textdomain($this->get_namespace(), false, dirname(plugin_basename(__FILE__)) . '/../languages/');

		//  If running in the admin, initialize plugin options in the admin
		if (is_admin()) {
			$this->_init_admin_options();
			$this->_add_options();
		}

		//  Set the last log time
		$this->last_log_time = microtime(true) * 1000000;

	}

	//  Install the plugin
	public function activate() {

		//  Create the database tables the plugin needs if necessary
		$this->_create_tables();

		//  If we need to migrate data, do it now
		$this->_migrate_data();

	}

	//  Uninstall the plugin -- Let's not delete anything
	public function deactivate() {
	}

	//  Add options
	protected function _add_options() {

		//  Loop through options and add them if the don't exist already
		foreach ($this->options as $group=>$properties) {

			//  Only proceed if the option doesn't exist already
			$optionExists = get_option($properties['key']);
			if (!$optionExists) {

				//  Add the option
				add_option($properties['key']);

				//  Loop through fields and update any defaults
				$values = array();
				foreach ($properties["fields"] as $option=>$field) {

					//  If there is a default, use it
					if (array_key_exists('default', $field)) {
						$values[$option] = $field['default'];

						//  Or else default to an empty string
					} else {
						$values[$option] = '';
					}

				}

				//  Update the option with initial values
				update_option($properties['key'], $values);

			}

		}

	}

	//  Remove all options
	protected function _remove_options() {
		foreach ($this->options as $group) {
			delete_option($group['key']);
		}
	}

	//  Create database tables if necessary
	protected function _create_tables() {

		//  Loop through tables and create them if necessary
		foreach ($this->tables as $table=>$fields) {

			//  Set the table name
			$tableName = $this->get_table($table);

			//  If the table doesn't exist, create it
			$tableExists = $this->db->query("SHOW TABLES LIKE '" . $tableName . "'");
			if (!$tableExists) {

				//  Start the SQL string
				$createSql = 'CREATE TABLE ' . $tableName . ' (';
				$primaryKey = false;

				//  Loop through defined fields
				foreach ($fields as $field=>$properties) {

					//  Add the field & type
					$createSql .= $field . ' ' . $properties['type'];

					//  Include the length if provided
					if (isset($properties['length'])) {
						$createSql .= '(' . $properties['length'] . ')';

						//  Otherwise if the field has predetermined options (e.g., ENUM), define those
					} else if (isset($properties['options'])) {
						$createSql .= '(';
						foreach ($properties['options'] as $option) {
							$createSql .= '\'' . $option . '\',';
						}
						$createSql = substr($createSql, 0, -1) . ')';
					}

					//  Add the NOT NULL flag if set
					if (isset($properties['null']) && !$properties['null']) {
						$createSql .= ' NOT NULL';
					} else if (!isset($properties['default'])) {
						$properties['default'] = 'NULL';
					}

					//  Add the default if set
					if (isset($properties['default'])) {
						$createSql .= ' DEFAULT ';
						if ($properties['default'] == 'NULL') {
							$createSql .= $properties['default'];
						} else {
							$createSql .= '\'' . $properties['default'] . '\'';
						}
					}

					//  If this is the first field, assume auto increment and primary key
					if (!$primaryKey) {
						$createSql .= ' AUTO_INCREMENT';
						$primaryKey = $field;
					}

					$createSql .= ', ';

				}

				//  Add the primary key (assumed to be first defined field)
				$createSql .= 'PRIMARY KEY (' . $primaryKey . ')) CHARACTER SET utf8 COLLATE utf8_general_ci';

				//  Make the query
				$this->db->query($createSql);

			}

		}

	}

	//  Migrate data on activation
	protected function _migrate_data() {}

	//  Initialize admin options
	protected function _init_admin_options() {

		//  Save the option fields
		$this->options = $this->get_config('options');

		//  Loop through options
		foreach ($this->options as $group=>$properties) {

			//  Add the section key as a property for later reference
			$this->options[$group]['key'] = $this->get_options_group_key($group);

			//  Loop through options
			foreach ($properties['fields'] as $option=>$field) {

				//  If the field has subfields, generate them programmatically
				if (array_key_exists('fields', $field)) {

					//  Loop through subfields
					foreach ($field['fields'] as $suboption=>$subfield) {

						//  If the subfield is not an array, use the value as the label and default
						if (!is_array($subfield)) {
							$subfield = array(
								'label'		=>	$subfield,
								'default'	=>	$subfield
							);
						}

						//  Loop through transferrable subfield properties and transfer them
						foreach ($this->transferableSubfieldProperties as $property) {
							if ((!array_key_exists($property, $subfield)) && array_key_exists($property, $field)) {
								$subfield[$property] = $field[$property];
							}
						}

						//  Set flag to not show subfield because parent field will do it
						$subfield['display'] = false;

						//  If a type is not set, default to text
						if (!array_key_exists('type', $subfield)) {
							if (array_key_exists('child_type', $field)) {
								$subfield['type'] = $field['child_type'];
							} else {
								$subfield['type'] = 'text';
							}
						}

						//  Add the new field
						$this->options[$group]['fields'][$option . $this->get_key_delimiter() . $suboption] = $subfield;

					}

				}

			}

		}

	}

	//  Display error messages
	public function display_errors($errors = array()) {
		if (!empty($errors)) {
			if (!is_array($errors)) {
				$errors = array($errors);
			}
			include($this->get_path('/views/public/errors.php'));
		}
	}

	//  Get the namespace
	public function get_namespace() {
		return $this->namespace;
	}

	//  Add the plugin prefix
	public function add_namespace($str, $delimiter = false) {
		if (!is_array($str)) {
			$str = array($str);
		}
		if (!$delimiter) {
			$delimiter = $this->get_key_delimiter();
		}
		$newStr = $this->get_namespace();
		foreach ($str as $tmpStr) {
			$newStr .= $delimiter . $tmpStr;
		}
		if ($delimiter) {
			$newStr = str_replace($this->get_key_delimiter(), $delimiter, $newStr);
		}
		return $newStr;
	}

	//  Strip out the namespace from a string
	public function strip_namespace($str, $delimiter = false) {
		return substr($str, strlen($this->add_namespace('', $delimiter)));
	}

	//  Get the key delimiter
	public function get_key_delimiter() {
		return $this->keyDelimiter;
	}

	//  Get configuration array
	public function get_config($group = null, $sub_group = null) {
		if( is_null($group) ) {
			return $this->config;
		}
		if( is_null($sub_group) ) {
			return $this->config[$group];
		}
		return $this->config[$group][$sub_group];
	}

	//  Get a data label
	public function get_data_label($table, $field, $serializedField = false) {
		if ($serializedField) {
			return $this->config['data_labels'][$table][$field][$serializedField];
		} else {
			return $this->config['data_labels'][$table][$field];
		}
	}

	//  Get option
	public function get_option($option, $group, $sub_group=null) {

		if( $sub_group ) {
			return $this->_get_sub_option($option, $group, $sub_group);
		}

		if (!isset($this->optionValues[$group])) {
			$this->optionValues[$group] = $this->get_options_group($group);
		}
		if (isset($this->optionValues[$group][$option])) {
			return $this->optionValues[$group][$option];
		} else {
			return '';
		}
	}

	/**
	 * Gets an option as a boolean.
	 *
	 * @param $name string - The option to lookup.
	 * @param null $group string - (optional) The group of the option.
	 * @param null $sub_group - (optional) The sub group of the option.
	 * @return bool
	 */
	public function get_bool_option($name, $group = null, $sub_group = null) {
		return $this->str_to_boolean(
			$this->get_option(
				$name,
				$group,
				$sub_group
			)
		);
	}

	//  Get integer option
	public function get_int_option($name, $group=null, $sub_group=null) {
		return intval( $this->get_option($name, $group, $sub_group) );
	}

	// Get option form sub-group
	private function _get_sub_option($option, $group, $sub_group){

		// Get wordpress setting if it exists
		$wp_group = get_option('administrate_'.$group.'_options');

		if( array_key_exists($sub_group.'_'.$option, $wp_group) ) {
			return $wp_group[$sub_group.'_'.$option];
		}

		// Get option default value
		$options = $this->config['options'];
		$options_group = $options[$group]['fields'];
		$options_sub = $options_group[$sub_group]['fields'];

		return $options_sub[$option]['default'];
	}

	//  Update an option
	public function update_option($option, $group, $value) {
		$values = $this->get_options_group($group);
		$values[$option] = $value;
		return update_option($this->get_options_group_key($group), $values);
	}

	//  Get options group key
	public function get_options_group_key($group) {
		return $this->add_namespace(array($group, 'options'));
	}

	//  Get all options
	public function get_options() {
		return $this->options;
	}

	//  Get a specific option field definition
	public function get_option_field($field, $group) {
		return $this->options[$group]['fields'][$field];
	}

	//  Get an entire section of options
	public function get_options_group($group) {
		return get_option($this->get_options_group_key($group));
	}

	//  Display the widget
	public function display_widget($key, $params = array()) {
		$this->controller->display_widget($key, $params);
	}

	//  Run the widget
	public function run_widget($key, $params = array()) {
		return $this->controller->run_widget($key, $params);
	}

	//  Get a table
	public function get_table($key) {
		return $this->tableMaps[$key];
	}

	//  Get the table field definitions
	public function get_table_fields($key) {
		return $this->tables[$key];
	}

	//  Get the plugin path
	public function get_path($path = '') {
		return $this->path . $path;
	}

	//  Get the plugin URL
	public function get_url($path = '') {
		return $this->url . $path;
	}

	/**
	 * Returns the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return string - The translated text.
	 */
	public function get_text($text) {
		return $text;
	}

	/**
	 * Echoes out the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return null
	 */
	public function echo_text($text) {
		echo $text;
	}

	//  Whether or not Javascript is enabled
	public function is_js_enabled() {
		return $this->jsEnabled;
	}

	//  Whether or not the current page is a plugin admin page
	public function is_admin_page() {
		if( is_admin() && isset($_GET['page']) ) {
			if( strstr($_GET['page'], $this->get_namespace()) ) {
				return true;
			}
		}
		return false;
	}

	//  Show an error
	public function show_error($str) {
		$this->_show_message($str, 'error');
	}

	//  Show a notice
	public function show_notice($str) {
		$this->_show_message($str, 'updated');
	}

	//  Show a message
	protected function _show_message($str, $type = 'updated') {
		array_push($this->messages, array('str'=>$str, 'type'=>$type));
	}

	//  Get messages
	public function get_messages() {
		return $this->messages;
	}

	//  Convert a string to a key-compatible string
	public function str_to_key($str) {
		return trim(preg_replace('/[^a-z0-9]+/', '_', strtolower($str)));
	}

	//  Parse the plugin readme
	public function parse_readme($path) {

		//  Get the file contents
		$lines = file($path);

		//  Initialize the array
		$result = array();

		//  Loop through lines
		$i = -1;
		foreach ($lines as $line) {

			//  Skip empty lines
			if (!empty($line)) {

				$line = trim($line);

				//  If the line starts with
				if (substr($line, 0, 4) == '=== ') {

					$result[++$i] = array();
					$result[$i]['title'] = substr($line, 4, strpos($line, ' ===', 4) - 4);
					$result[$i]['items'] = array();
					$top = true;
					$sub = false;

				} else if (substr($line, 0, 3) == '== ') {

					$n = count($result[$i]['items']);
					$result[$i]['items'][$n] = array();
					$result[$i]['items'][$n]['title'] = substr($line, 3, strpos($line, ' ==', 3) - 3);
					$result[$i]['items'][$n]['items'] = array();
					$top = false;
					$sub = true;

				} else if (substr($line, 0, 2) == '= ') {

					$d = count($result[$i]['items'][$n]['items']);
					$result[$i]['items'][$n]['items'][$d] = array();
					$result[$i]['items'][$n]['items'][$d]['title'] = substr($line, 2, strpos($line, ' ==', 2) - 2);
					$result[$i]['items'][$n]['items'][$d]['items'] = array();
					$top = false;
					$sub = false;

					//  Or else add a line to the current item
				} else {
					if ($top) {
						array_push($result[$i]['items'], $line);
					} else if ($sub) {
						array_push($result[$i]['items'][$n]['items'], $line);
					} else {
						array_push($result[$i]['items'][$n]['items'][$d]['items'], $line);
					}
				}

			}

		}

		//  Return the result
		return $result;

	}

	// Deprecated
	public function to_boolean($str) {
		return ( intval($str) === 1 );

	}

	//  Convert string to boolean
	public function str_to_boolean($bool_str) {
		return (
			( $bool_str === true ) ||
			( intval($bool_str) === 1 ) ||
			( strtolower($bool_str) === 'true' )
		);
	}

	//  Pluralize a word
	public function pluralize_str($num, $singular, $plural = false) {
		if ($num == 1) {
			return $singular;
		} else {
			if ($plural) {
				return $plural;
			} else {
				if (substr($singular, -1) == 'y') {
					return $substr($singular, 0, -1) . 'ies';
				} else if (substr($singular, -1) == 'x') {
					return $singular . 'es';
				} else {
					return $singular . 's';
				}
			}
		}
	}

}
