<?php

//  Authorize.Net Checkout Payment Processor
class AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet extends AdministrateWidgetCheckoutPaymentProcessor {

	const PRODUCTION_API_URL   = 'https://secure.authorize.net/gateway/transact.dll';
	const SANDBOX_API_URL      = 'https://test.authorize.net/gateway/transact.dll';
	private $is_sandbox         = true;

	//  Constructor
	public function __construct(&$plugin, &$widget, &$order = false) {
		$this->display_name = "Authorize.Net";
		
		//  Call parent class
		parent::__construct($plugin, $widget, $order);

		// Initialize API
		//
		$this->is_sandbox = (bool)($this->plugin->get_option('authorizenet_mode', 'checkout') == "sandbox");
		$this->api_url = $this->is_sandbox ?
			AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet::SANDBOX_API_URL :
			AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet::PRODUCTION_API_URL;

		$this->api_login_id     = $this->plugin->get_option('authorizenet_user', 'checkout');
		$this->transaction_key  = $this->plugin->get_option('authorizenet_key', 'checkout');
		$this->transaction_md5  = $this->plugin->get_option('authorizenet_md5', 'checkout');

		//  If the order isn't defined yet and an item number is in the request, initialize order
		if (!$this->order && isset($_REQUEST['x_cust_id'])) {
			$this->set_order($this->get_order());
		}
	}

	//  Get the order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$where = array(
				'order_id' => $_REQUEST['x_cust_id']
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->plugin, $where);
		}
		return $this->order;
	}

	/**
	 * Returns a hash representingthe transaction for verification on relay response.
	 *
	 * @param $login_id string - authorize.NET login ID
	 * @param $sequence string - order number
	 * @param $timestamp integer - timestamp
	 * @param $amount float - amount
	 * @param $currency string - 3-character currency code
	 * @param $key - authorize.NET api key
	 * @return string - the completed hash
	 */
	private static function _transaction_hash(
		$login_id, $sequence,
		$timestamp, $amount,
		$currency, $key
	) {
		$hash_string =  $login_id . "^" . $sequence . "^" . $timestamp . "^" . $amount . "^" . $currency;
		return ( function_exists('hash_hmac') ) ?
			hash_hmac("md5", $hash_string, $key) :
			bin2hex(mhash(MHASH_MD5, $hash_string, $key));
	}

	// Prepare parameter values to be passed on to the Authorize.net API
	private function _prepare_api_parameters() {

		global $_ADMINISTRATE_CONFIG;

		// The API key and currency are required for the relay response endpoint.
		$get_params = array(
			'api' => $this->api_login_id,
			'currency' => $this->order->get_currency()
		);

		$relay_response_url = $_ADMINISTRATE_CONFIG['plugin']['url']
			. 'common/processors/authorize/relay_response_wordpress.php?'
			. http_build_query($get_params);

		// Transaction parameters
		//
		$transaction = array(
			"x_login" => $this->api_login_id,
			"x_version" => "3.1",
			"x_test_request" => "false",
			"x_show_form" => "PAYMENT_FORM",

			"x_relay_response" => "true",
			"x_delim_data" => "false",
			"x_relay_url" => $relay_response_url,

			"x_cancel_url" =>
				$this->widget->get_step_url(constant('ADM_CHECKOUT_STEP_COURSE'), true),
			"x_cancel_url_text" => __("Cancel")
		);

		// Purchase information
		//
		$purchase = array(
			"x_method" => "CC",
			"x_po_num" => $this->order->get_session_id(),
			"x_cust_id" => $this->order->get_id(),

			'x_currency_code'   => $currency = $this->order->get_currency(),
			'x_invoice_num'     => date('YmdHis')
		);

		if($this->order->get_event_id()) {
			$mapped_items = $this->order->get_mapped_items_and_prices();
			$item_amount = 0;

			foreach($mapped_items as $item) {
				$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
			}

			$purchase['x_amount'] =
				(($this->order->get_event_gross_price() * $this->order->get_num_attendees()) + $item_amount);

			$purchase['x_description'] =
				$this->order->get_num_attendees() . __(" Course Booking(s), ") . $this->order->get_course_title();

		} else {
			$track = $this->order->get_track();
			list(
				$tax_amount,
				$tax_name,
				$net_price,
				$discounted_net_price,
				$gross_price,
				$discounted_gross_price,
				) = AdministrateLearningTrackWidget::price_detail(
				$track,
				$this->service->get_option('currency', 'pricing'),
				$this->order->get_num_attendees(),
				$this->get_order()->get_discount_type(),
				$this->get_order()->get_discount_rate()
			);

			$purchase['x_amount'] = $discounted_gross_price;
			$purchase['x_description'] = $track->description;
		}

		// Random transaction sequence number, timestamp and hash
		//
		$sequence = rand(1, 1024);
		$timestamp = time();
		$hash = array(
			"x_fp_sequence"  => $sequence,
			"x_fp_timestamp" => $timestamp,
			"x_fp_hash" => AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet::_transaction_hash(
					$this->api_login_id,
					$sequence,
					$timestamp,
					$purchase['x_amount'],
					$purchase['x_currency_code'],
					$this->transaction_key
				)
		);

		// Customer details
		//
		$country_id = $this->order->get_invoice_country();
		$countries = new AdministrateCountriesList();
	
		$customer = array(
			"x_first_name" => $this->order->get_buyer_first_name(),
			"x_last_name" => $this->order->get_buyer_last_name(),
			"x_company" => $this->order->get_buyer_company(),

			"x_address" => $this->order->get_invoice_street(),
			"x_city" => $this->order->get_invoice_city(),
			"x_zip" => $this->order->get_invoice_postal_code(),    // Postcode / ZIP
			"x_state" =>  $this->order->get_invoice_territory(),   // State / County
			"x_country" => $countries->get_name($country_id), // Country

			"x_phone" => $this->order->get_buyer_phone(),
			"x_email" => $this->order->get_buyer_email(),

			"x_ship_to_first_name" => $this->order->get_buyer_first_name(),
			"x_ship_to_last_name" => $this->order->get_buyer_last_name(),
			"x_ship_to_company" => $this->order->get_buyer_company(),
			"x_ship_to_address" => $this->order->get_invoice_street(),
			"x_ship_to_city" => $this->order->get_invoice_city(),
			"x_ship_to_zip" => $this->order->get_invoice_postal_code(),    // Postcode / ZIP
			"x_ship_to_state" =>  $this->order->get_invoice_territory(),   // State / County
			"x_ship_to_country" => $countries->get_name($country_id), // Country
		);

		// Return complete parameter list
		return array_merge($transaction,$purchase,$customer,$hash);
	}

	// Is order key present in the request
	public static function has_order_key() {
		return isset( $_REQUEST['x_cust_id'] );
	}

	//  Display order form
	public function display_form() {
		// Build API parameters
		$params = $this->_prepare_api_parameters();

		// Authorize.Net payment button
		$this->_form_end();
		$this->_form_begin($this->api_url);
		foreach( $params as $name => $val) $this->_form_parameter($name, $val);
		$this->_form_button(__("Place Order by Credit Card"));
	}

	// Authorize.net form generation (API parameters and submit button)
	private static function _form_begin($action_url) {
		echo '<form method="post" class="cc_submit_form" action="'.htmlspecialchars($action_url).'" name="pay_form">';
	}
	private static function _form_end() {
		echo '</form>';
	}
	private static function _form_parameter($name,$value){
		echo '<input type="hidden" name="'.$name.'" value="'.$value.'"/>';
	}
	private static function _form_button($text) {
		global $_ADMINISTRATE;
		echo '<input type="submit" value="'.__($text, 'administrate').'" class="'.$_ADMINISTRATE->add_namespace('btn', '-').'">';
	}
}
